@@ -80,8 +80,8 @@
 
 #define READ_END 0
 #define WRITE_END 1
-#define USERNAME_MAX		127
-#define PATH_MAX		256
+#define USERNAME_MAX    127
+#define PATH_MAX    256
 #define BUFSIZE ((USERNAME_MAX +1) + ((PATH_MAX+1) * 2))
 #define CONFIGFILE     "/etc/security/pam_encfs.conf"
 
@@ -237,6 +237,24 @@ int buildCmd(char *arg[], int pos, char *line)
     return pos - orig_pos;
 }
 
+
+const char *getHome(struct passwd *pwd, pam_handle_t * pamh)
+{
+    const char *tmp = NULL;
+
+    tmp = pam_getenv(pamh, "HOME");
+    
+    if (!tmp || *tmp == '\0')
+    {
+        if (pwd->pw_dir && pwd->pw_dir != '\0')
+            return pwd->pw_dir;
+        else
+            return NULL;
+    }
+    else
+        return tmp;
+}
+
 int readconfig(struct passwd *pwd, pam_handle_t * pamh, const char *user,
                char *path, char *targetpath, char *encfs_options,
                char *fuse_options)
@@ -245,7 +263,7 @@ int readconfig(struct passwd *pwd, pam_handle_t * pamh, const char *user,
     char line[BUFSIZE];
     char username[USERNAME_MAX];
     int parsed;
-    char *tmp;
+    const char *tmp;
 
     // Return 1 = error, 2 = silent error (ie already mounted)
 
@@ -294,32 +312,50 @@ int readconfig(struct passwd *pwd, pam_handle_t * pamh, const char *user,
             searchAndReplace(encfs_options);
             
             // Check if this is the right user / default user.
-            if ((strcmp("-",username) != 0) && (strcmp(user,username) != 0))
-            	continue;
+            if ((strcmp("-",username) != 0) && (strcmp(user,username) != 0)
+                && (strcmp("*",username) !=0))
+              continue;
             
             
             if (strcmp("-",username) == 0) {
-            	strcat(path, "/");
+              strcat(path, "/");
                 strcat(path, user);
                 // Todo check if dir exists and give better error msg.
             }
             
+            // If username is '*', paths are relative to $HOME
+            if (strcmp("*", username) == 0
+                && strcmp("-", targetpath) != 0)
+            {
+                if ((tmp = getHome(pwd, pamh)))
+                {
+                        char home[PATH_MAX];
+
+                        strcpy(home, tmp);
+                        strcat(home, "/");
+                        strcat(home, path);
+                        strcpy(path, home);
+
+                        strcpy(home, tmp);
+                        strcat(home, "/");
+                        strcat(home, targetpath);
+                        strcpy(targetpath, home);
+                }
+            }
+            
             if (strcmp("-", targetpath) == 0)
             {
                 // We do not have targetpath, construct one.
                 strcpy(targetpath, "");
-                tmp = pam_getenv(pamh, "HOME");
-                if (tmp && *tmp != '\0')
+
+                if ((tmp = getHome(pwd, pamh)))
                 {
                     strcpy(targetpath, tmp);
                 }
-                else if (pwd->pw_dir && pwd->pw_dir != '\0')
-                {
-                    strcpy(targetpath, pwd->pw_dir);
-                }
             }
 
 
+
             // Done, check targetpath and return.
 
             if (!targetpath || *targetpath == '\0')
@@ -353,7 +389,8 @@ int readconfig(struct passwd *pwd, pam_handle_t * pamh, const char *user,
 
             // Path does not exist, if we're a specified user give error, if not keep looking.
 
-            if (strcmp("-", username) != 0)
+            if ((strcmp("-", username) != 0)
+              && (strcmp("*", username) != 0))
             {
                 _pam_log(LOG_ERR, "Path for %s does not exist (%s)", user,
                          path);
@@ -582,7 +619,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t * pamh,
     int exitstatus = WEXITSTATUS(status);
     char buff[512];
 
-    len = read(inpipe[READ_END], &buff, 512);
+    len = read(inpipe[READ_END], &buff, 511);
     close(inpipe[READ_END]);
     buff[len] = 0;
     if (!checkmnt(targetpath) && (len > 0 || exitstatus > 0))
@@ -639,7 +676,7 @@ PAM_EXTERN int pam_sm_close_session(pam_handle_t * pamh,
     }
 
     args[0] = "fusermount";
-    args[1] = "-u";
+    args[1] = "-uz";
     args[2] = targetpath;
     args[3] = NULL;
 
@@ -657,6 +694,9 @@ PAM_EXTERN int pam_sm_close_session(pam_handle_t * pamh,
             exit(127);
     }
 
+    if (waitpid(pid, NULL, 0) == -1)
+      _pam_log(LOG_ERR, "Waitpid failed - %s", strerror(errno));
+
     /*We'll get this error every single time we have more than one session active, todo fix this with some better checks + support fuser -km if no more session connected.  
        if (checkmnt(targetpath)) {
        _pam_log(LOG_ERR,"Failed to unmount %s",targetpath);
